"""Unit Tests for the module"""
import logging

from django.test import TransactionTestCase

LOGGER = logging.getLogger(name="django-iubenda")


class IubendaTestCase(TransactionTestCase):
    """Test Case for django-iubenda"""

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("Tests setUp")

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("Tests tearDown")



import sys

import pytest


def split_class_and_function(string):
    class_string, function_string = string.split(".", 1)
    return f"{class_string} and {function_string}"


def is_function(string):
    # `True` if it looks like a test function is included in the string.
    return string.startswith("test_") or ".test_" in string


def is_class(string):
    # `True` if first character is uppercase - assume it's a class name.
    return string[0] == string[0].upper()


if __name__ == "__main__":
    if len(sys.argv) > 1:
        pytest_args = sys.argv[1:]
        first_arg = pytest_args[0]

        try:
            pytest_args.remove("--coverage")
        except ValueError:
            pass
        else:
            pytest_args = [
                "--cov",
                ".",
                "--cov-report",
                "xml",
            ] + pytest_args

        if first_arg.startswith("-"):
            # `runtests.py [flags]`
            pytest_args = ["tests"] + pytest_args
        elif is_class(first_arg) and is_function(first_arg):
            # `runtests.py TestCase.test_function [flags]`
            expression = split_class_and_function(first_arg)
            pytest_args = ["tests", "-k", expression] + pytest_args[1:]
        elif is_class(first_arg) or is_function(first_arg):
            # `runtests.py TestCase [flags]`
            # `runtests.py test_function [flags]`
            pytest_args = ["tests", "-k", pytest_args[0]] + pytest_args[1:]
    else:
        pytest_args = []

    sys.exit(pytest.main(pytest_args))



from modeltranslation.translator import TranslationOptions, register

from .models import Iubenda


@register(Iubenda)
class IubendaTranslationOptions(TranslationOptions):
    fields = ("iub_policy_id",)



"""Unit Tests for the module"""
import logging

from django.test import TransactionTestCase, override_settings

LOGGER = logging.getLogger(name="django-iubenda")


class IubendaTestCase(TransactionTestCase):
    """Test Case for django-iubenda"""

    fixtures = ["data.json"]

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("Tests setUp")

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("Tests tearDown")

    def test_privacy_template_with_compress(self):
        """Test that render Privacy page using compress."""
        LOGGER.debug("Render Privacy page")
        response = self.client.get("/privacy-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, "iubenda/privacy-compress.html")

    def test_cookie_template_with_compress(self):
        """Test that render Cookie page using compress."""
        LOGGER.debug("Render Cookie page")
        response = self.client.get("/cookie-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, "iubenda/cookie-compress.html")

    @override_settings(IUBENDA_USE_COMPRESS=False)
    def test_privacy_template(self):
        """Test that render Privacy page."""
        LOGGER.debug("Render Privacy page")
        response = self.client.get("/privacy-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, "iubenda/privacy.html")

    @override_settings(IUBENDA_USE_COMPRESS=False)
    def test_cookie_template(self):
        """Test that render Cookie page."""
        LOGGER.debug("Render Cookie page")
        response = self.client.get("/cookie-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)
        self.assertTemplateUsed(response, "iubenda/cookie.html")



"""example URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Add an import:  from blog import urls as blog_urls
    2. Import the include() function: from django.conf.urls import url, include
    3. Add a URL to urlpatterns:  url(r'^blog/', include(blog_urls))
"""
from django.urls import path

from .views import cookie, privacy

urlpatterns = [
    path("privacy-policy/", privacy, name="privacy"),
    path("cookie-policy/", cookie, name="cookie"),
]



import json

import requests
from django.conf import settings
from django.contrib import messages
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.shortcuts import render
from django.views.decorators.http import require_http_methods
from django.views.decorators.vary import vary_on_headers

from .models import Iubenda


@require_http_methods(["GET"])
@vary_on_headers("User-Agent", "Cookie")
def privacy(request):
    context = {}
    context_cache = {}

    cache_key = f"api_iubenda_privacy_{request.LANGUAGE_CODE}"
    try:
        context_cache = cache.get(cache_key)
    except Exception as err:
        messages.warning(request, err)

    if not context_cache or context_cache is None:
        try:
            iubenda = (
                Iubenda.objects.filter(site=Site.objects.get_current())
                .values("iub_policy_id")
                .get()
            )
            r = requests.get(
                f'https://www.iubenda.com/api/privacy-policy/{iubenda["iub_policy_id"]}',
                params=request.GET,
            )
            if r.status_code == 200:
                context = {"req_privacy": json.loads(r.content)}

            cache.set(cache_key, context, timeout=86400)
            context_cache = cache.get(cache_key)

        except Exception as err:
            messages.error(request, err)
    if getattr(settings, "IUBENDA_USE_COMPRESS", True):
        return render(request, "iubenda/privacy-compress.html", context_cache)
    return render(request, "iubenda/privacy.html", context_cache)


@require_http_methods(["GET"])
@vary_on_headers("User-Agent", "Cookie")
def cookie(request):
    context = {}
    context_cache = {}

    cache_key = f"api_iubenda_cookie_{request.LANGUAGE_CODE}"
    try:
        context_cache = cache.get(cache_key)
    except Exception as err:
        messages.warning(request, err)

    if not context_cache or context_cache is None:
        try:
            iubenda = (
                Iubenda.objects.filter(site=Site.objects.get_current())
                .values("iub_policy_id")
                .get()
            )
            r = requests.get(
                f'https://www.iubenda.com/api/privacy-policy/{iubenda["iub_policy_id"]}/cookie-policy',
                params=request.GET,
            )
            if r.status_code == 200:
                context = {"req_cookie": json.loads(r.content)}

            cache.set(cache_key, context, timeout=86400)
            context_cache = cache.get(cache_key)

        except Exception as err:
            messages.error(request, err)
    if getattr(settings, "IUBENDA_USE_COMPRESS", True):
        return render(request, "iubenda/cookie-compress.html", context_cache)
    return render(request, "iubenda/cookie.html", context_cache)



# Generated by Django 3.2.12 on 2022-04-27 19:36

import django.db.models.deletion
from django.db import migrations, models


def load_data_from_fixture(apps, schema_editor):
    from django.core.management import call_command

    call_command("loaddata", "data")


def delete_data(apps, schema_editor):
    Iubenda = apps.get_model("iubenda", "Iubenda")
    Iubenda.objects.all().delete()


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("sites", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Iubenda",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "iub_site_id",
                    models.IntegerField(unique=True, verbose_name="Iubenda Site ID"),
                ),
                (
                    "iub_policy_id",
                    models.IntegerField(unique=True, verbose_name="Iubenda Policy ID"),
                ),
                (
                    "iub_policy_id_it",
                    models.IntegerField(
                        null=True, unique=True, verbose_name="Iubenda Policy ID"
                    ),
                ),
                (
                    "iub_policy_id_en",
                    models.IntegerField(
                        null=True, unique=True, verbose_name="Iubenda Policy ID"
                    ),
                ),
                ("active", models.BooleanField(default=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sites.site",
                        verbose_name="Django Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Iubenda Policy",
                "verbose_name_plural": "Iubenda Policies",
            },
        ),
        # migrations.RunPython(load_data_from_fixture, delete_data),
    ]



from django.contrib import sitemaps
from django.urls import reverse


class PrivacySitemap(sitemaps.Sitemap):
    priority = 0.8
    changefreq = "monthly"

    def items(self):
        return ["privacy"]

    def location(self, item):
        return reverse(item)


class CookieSitemap(sitemaps.Sitemap):
    priority = 0.8
    changefreq = "monthly"

    def items(self):
        return ["cookie"]

    def location(self, item):
        return reverse(item)



"""Test's settings"""
import os

from django.utils.translation import gettext_noop

DEBUG = False

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]

SECRET_KEY = "NOTASECRET"

ALLOWED_HOSTS = ["*"]

SITE_ID = 1

INSTALLED_APPS = [
    "modeltranslation",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sites",
    "django.contrib.sitemaps",
    "compressor",
    "iubenda",
]

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "DIRS": [os.path.join(BASE_DIR, "templates")],
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.media",
                "iubenda.context_processors.iubenda",
            ],
        },
    }
]

# Compress
COMPRESS_ENABLED = True
COMPRESS_OFFLINE = True
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder",
)
COMPRESS_ROOT = "/staticroot"


ROOT_URLCONF = "tests.urls"

MIDDLEWARE = (
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.cache.UpdateCacheMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.cache.FetchFromCacheMiddleware",
)

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}

USE_TZ = True
LANGUAGE_CODE = "en"
USE_I18N = True

STATIC_URL = "/static/"

# Languages we provide translations for, out of the box.
LANGUAGES = [
    ("en", gettext_noop("English")),
    ("it", gettext_noop("Italian")),
]



from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import gettext_lazy as _


class Iubenda(models.Model):
    site = models.ForeignKey(
        Site, verbose_name=_("Django Site"), on_delete=models.CASCADE
    )
    iub_site_id = models.IntegerField(verbose_name=_("Iubenda Site ID"), unique=True)
    iub_policy_id = models.IntegerField(
        verbose_name=_("Iubenda Policy ID"), unique=True
    )
    active = models.BooleanField(default=True)

    class Meta:
        verbose_name = _("Iubenda Policy")
        verbose_name_plural = _("Iubenda Policies")



"""
Context processors for Iubenda app
"""

from django.conf import settings
from django.contrib import messages
from django.contrib.sites.models import Site
from django.core.cache import cache

from .models import Iubenda


def iubenda(request):
    """Returns the info for:
    - iub_site_id: Iubenda Site ID
    - iub_policy_id: Iubenda Policy ID
    """

    cache_key = f"iubenda_{request.LANGUAGE_CODE}"
    context_cache = {}
    try:
        context_cache = cache.get(cache_key)
    except Exception as err:
        messages.warning(request, err)
        context_cache = {}

    if not context_cache or context_cache is None:
        try:
            context = {
                "cx_iubenda": Iubenda.objects.filter(site=Site.objects.get_current())
                .values("iub_site_id", "iub_policy_id")
                .get(),
            }

            cx_iubenda_options = getattr(settings, "IUBENDA_OPTIONS", False)
            if cx_iubenda_options:
                context.update({"cx_iubenda_options": cx_iubenda_options})

            cx_iubenda_gtm = getattr(settings, "IUBENDA_GTM", False)
            if cx_iubenda_gtm:
                context.update({"cx_iubenda_gtm": cx_iubenda_gtm})

            cx_iubenda_nonce = getattr(settings, "IUBENDA_CSP_NONCE", False)
            if cx_iubenda_nonce:
                context.update({"cx_iubenda_nonce": cx_iubenda_nonce})

            context_cache = cache.set(cache_key, context, timeout=86400)
            return context
        except Exception as err:
            messages.error(request, err)
    return context_cache



"""Test's urls view for django-iubenda"""
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.contrib.sitemaps.views import sitemap
from django.urls import include, path, re_path
from django.views.i18n import JavaScriptCatalog

from iubenda.sitemaps import CookieSitemap, PrivacySitemap

sitemaps = {
    "privacy": PrivacySitemap,
    "cookie": CookieSitemap,
}

urlpatterns = [
    path("", include("iubenda.urls")),
    path("admin/", admin.site.urls),
]

urlpatterns += i18n_patterns(
    re_path(r"^jsi18n/$", JavaScriptCatalog.as_view(), name="javascript-catalog"),
    path("", include("iubenda.urls")),
    path(
        "sitemap.xml",
        sitemap,
        {"sitemaps": sitemaps},
        name="django.contrib.sitemaps.views.sitemap",
    ),
)



"""Unit Tests for the module"""
import logging

from django.test import TransactionTestCase

LOGGER = logging.getLogger(name="django-iubenda")


class IubendaTestCase(TransactionTestCase):
    """Test Case for django-iubenda"""

    fixtures = ["data.json"]

    def setUp(self):
        """Set up common assets for tests"""
        LOGGER.debug("Tests setUp")

    def tearDown(self):
        """Remove Test Data"""
        LOGGER.debug("Tests tearDown")

    def test_sitemap(self):
        """Test that render sitemap.xml."""
        LOGGER.debug("Render Sitemap")
        response = self.client.get("/it/sitemap.xml", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)

    def test_privacy_urls(self):
        """Test that render Privacy page."""
        LOGGER.debug("Render Privacy page")
        response = self.client.get("/privacy-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)

    def test_cookie_urls(self):
        """Test that render Cookie page."""
        LOGGER.debug("Render Cookie page")
        response = self.client.get("/cookie-policy/", follow=True)
        LOGGER.debug(response)
        self.assertEqual(200, response.status_code)



from django.contrib import admin
from modeltranslation.admin import TranslationAdmin

from .models import Iubenda


@admin.register(Iubenda)
class IubendaAdmin(TranslationAdmin, admin.ModelAdmin):
    list_display = ["site", "iub_site_id", "iub_policy_id"]
    list_display_links = ["site"]
    ordering = ["site", "iub_site_id", "iub_policy_id"]
    search_fields = ["site"]
